use crate::value::{decoder, encoder, invert_map, AValue};
use decoder::{Decoder, DecodePlaceholderMap};
use encoder::{Encoder, EncodePlaceholderMap};
use std::io::{Read, Write};

pub type Error = decoder::Error;
pub type Result<T> = decoder::Result<T>;

pub struct Codec {
    pub decode_placeholders: Option<DecodePlaceholderMap>,
    pub encode_placeholders: Option<EncodePlaceholderMap>,
}

impl Codec {
    pub fn new(decode_placeholders: DecodePlaceholderMap) -> Self {
        let encode_placeholders = Some(invert_map(&decode_placeholders));
        Codec { decode_placeholders: Some(decode_placeholders), encode_placeholders }
    }

    pub fn without_placeholders() -> Self {
        Codec { decode_placeholders: None, encode_placeholders: None }
    }

    pub fn decoder<'a, R: Read>(&'a self, read: R) -> Decoder<'a, R> {
        Decoder::new(read, self.decode_placeholders.as_ref())
    }

    pub fn encoder<'a, 'w, W: Write>(&'a self, write: &'w mut W) -> Encoder<'w, 'a, W> {
        Encoder::new(write, self.encode_placeholders.as_ref())
    }

    pub fn decode<R: Read>(&self, read: R) -> Result<AValue> {
        self.decoder(read).next()
    }

    pub fn encode_bytes(&self, v: &AValue) -> Result<Vec<u8>> {
        let mut buf: Vec<u8> = Vec::new();
        self.encoder(&mut buf).write(v)?;
        Ok(buf)
    }
}
